import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
from PyVisualizer.CommonUtilities import CommonUtilities


commonUtils = CommonUtilities()
conn = commonUtils.connectAndAutorizeToServiceAccount("./service_account.json")
sheet1 = commonUtils.read_google_sheet(clientConnectionObject=conn,sGoogleSheetFileName="Copy_of_Greendeck_SE_Assignment_Task_2", sWorkSheetName="Sheet1")
sheet1_data = sheet1.get_all_records()
print(sheet1_data)
df = pd.DataFrame(sheet1_data)
class Visualizer():

    def barPlotSalesByYear(self, pandasDataframe, xColName, yColName, sSaveWithFileName):
        """
        Helps in plotting a bar plot, Avg Sales by year on y axis and year on x axis
        :param dataFrame: accepts a pandas dataframe
        :param xColName: Column name to be plotted on x axis
        :param yColName: Column name to be plotted on y axis
        :param sSaveWithFileName: It will save the file in present working directory with the mentioned file name
        """
        pandasDataframe['year'] = pandasDataframe[xColName].apply(lambda x: pd.Timestamp(x, unit='s').date().year)
        sns.barplot(x='year', y=yColName, data=pandasDataframe)
        plt.ticklabel_format(style='plain', axis='y')
        plt.title("Avg Sales per year")
        plt.xlabel("Year(s)", fontsize=10)
        plt.ylabel("Average_sales", fontsize=10)
        plt.xticks(rotation=45)
        plt.show()
        plt.savefig(sSaveWithFileName)

    def barPlotSalesByMonth(self, pandasDataframe, xColName, yColName, sSaveWithFileName):
        """
        Helps in plotting a bar plot, Avg Sales by month on y axis and year on x axis
        :param dataFrame: accepts a pandas dataframe
        :param xColName: Column name to be plotted on x axis
        :param yColName: Column name to be plotted on y axis
        :param sSaveWithFileName: It will save the file in present working directory with the mentioned file name
        """
        pandasDataframe['month'] = pandasDataframe[xColName].apply(
            lambda x: pd.Timestamp(x, unit='s').date().strftime("%B"))
        sns.barplot(x='month', y=yColName, data=pandasDataframe)
        plt.ticklabel_format(style='plain', axis='y')
        plt.title("Avg Sales by months")
        plt.xlabel("Months", fontsize=10)
        plt.ylabel("Average_sales", fontsize=10)
        plt.xticks(rotation=45)
        plt.show()
        plt.savefig(sSaveWithFileName)

    def getSalesByYearsAndMonths(self, pandasDataFrame):
        return pd.pivot_table(data=pandasDataFrame, values='average_sales', index=['year'], columns=['month'],
                              aggfunc='sum').T

# Visualizer().barPlotSalesByMonth(pandasDataframe=df,xColName="timestamp",yColName="average_sales",sSaveWithFileName="salesByMonth.png")


import gspread
from oauth2client.service_account import ServiceAccountCredentials

class CommonUtilities:

    def connectAndAutorizeToServiceAccount(self, serviceAccountFilePath):
        """
        This method will connect to google service account which is configured in service_account.json file.
        If successfully connected the it will return client connection obj
        """
        scope = ["https://spreadsheets.google.com/feeds", 'https://www.googleapis.com/auth/spreadsheets',
                 "https://www.googleapis.com/auth/drive.file", "https://www.googleapis.com/auth/drive"]
        try:
            creds = ServiceAccountCredentials.from_json_keyfile_name(serviceAccountFilePath, scope)
            connectedClientObj = gspread.authorize(creds)
            # self.connectedClientObj = gspread.service_account("./service_account.json", scopes=scope)
            # connectedClientObj = gspread.service_account(serviceAccountFilePath)
            if type(connectedClientObj) is gspread.client.Client:
                print("Successfully Authorized and Connected to Google Service Account. :)")
                return connectedClientObj
            else:
                print("Failed to connect and authorize to service account.")
                return None
        except Exception as e:
            print("Exception caught in Connecting and Authorizing Google Service Account : ", e)
            return None

    def read_google_sheet(self, clientConnectionObject, sGoogleSheetFileName, sWorkSheetName):
        """
        This method is used to read the google sheet file and worksheet inside it
        :param clientConnectionObject: Accepts a connection object to google account
        :param sGoogleSheetFileName: Accepts Google sheet file name
        :param sWorkSheetName: Accepts what worksheet needs to accessed from the file
        :return: Read the specified sheet and Returns the object of that sheet, if any exception then return None
        """

        # An empty string counts as False, variable with a value in it count as True
        if sGoogleSheetFileName and sWorkSheetName:
            try:
                sheets = clientConnectionObject.open(sGoogleSheetFileName)
                return sheets.worksheet(sWorkSheetName)
            except:
                raise Exception("Exception caught in reading the GoogleSheetFile/Worksheet.")
                return None
        else:
            raise ValueError("Parameter value cannot be empty.")
            return None



from setuptools import setup

def readme():
    with open('README.md') as f:
        README = f.read()
    return README

setup(
    name="PyVisualizer",
    version="1.0.7",
    description="A simple static visualizer.",
    long_description=readme(),
    long_description_content_type="text/markdown",
    url="https://github.com/vishalkashyap95/PyVisualizer_Task2.git",
    author="Vishal Kashyap",
    author_email="vkashyap569@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    packages=["PyVisualizer"],
    include_package_data=True,
    install_requires=["gspread","pandas","requests","matplotlib","seaborn"],
    entry_points={
        "console_scripts": [
            "PyVisualizer=PyVisualizer.Visualizer:__init__",
        ]
    },
)